/* vim: set sw=3 ts=3 sts=3 expandtab: */
#include "ecdb.h"
#include <fnmatch.h>

static void ecdb_handle_typebuf(Evas_Object *gui);
static void _button_cb_burn_data(void *data, Evas_Object *o,
      const char *emission, const char *source);
static void _button_cb_burn_audio(void *data, Evas_Object *o,
      const char *emission, const char *source);
static void _button_cb_burn_image(void *data, Evas_Object *o,
      const char *emission, const char *source);
static void _button_cb_erase(void *data, Evas_Object *o,
      const char *emission, const char *source);
static void ecdb_filelist_show(void);
static void ecdb_page_hide(const char *pname);
static void ecdb_filelist_focus_handle(int action, Evas_Object *fl);

static void
ecdb_cb_enter(Ecore_Evas *ee)
{
   Evas_Object *gui;
   gui = evas_object_name_find(ecore_evas_get(ee), "gui");
   edje_object_signal_emit(gui, "ecdb,window,enter", "ecdb");
}

static void
ecdb_cb_leave(Ecore_Evas *ee)
{
   Evas_Object *gui;
   gui = evas_object_name_find(ecore_evas_get(ee), "gui");
   edje_object_signal_emit(gui, "ecdb,window,exit", "ecdb");
}

static void
ecdb_cb_resize(Ecore_Evas *ee)
{
   int w, h;
   Evas_Object *gui;

   ecore_evas_geometry_get(ee, NULL, NULL, &w, &h);
   gui = evas_object_name_find(ecore_evas_get(ee), "gui");
   evas_object_resize(gui, w, h);
}

static void ecdb_filelist_focus_handle(int action, Evas_Object *fl)
{
   static Evas_Object *old_focus;
   Evas_Object *gui;

   if (action == IN)
   {
      old_focus = evas_focus_get(ecore_evas_get(em->main_win_ee));
      edje_object_signal_emit(old_focus, "ecdb,focus,out", "ecdb");
      evas_object_focus_set(fl, 1);
   }
   else if (old_focus)
   {
      edje_object_signal_emit(old_focus, "ecdb,focus,in", "ecdb");
      evas_object_focus_set(old_focus, 1);
      old_focus = NULL;

      gui = evas_object_name_find(ecore_evas_get(em->main_win_ee), "gui");
      edje_object_signal_emit(gui, "ecdb,filelist_overlay,deactivate", "ecdb");
      edje_object_part_text_set(gui, "filelist_overlay_text", NULL);
   }
}

static void
_cb_filelist_mouse_in(void *data, Evas *e __UNUSED__, Evas_Object *eo,
																	void *ev_data __UNUSED__)
{
   ecdb_filelist_focus_handle(IN, eo);
}

static void
_cb_filelist_mouse_out(void *data, Evas *e __UNUSED__, Evas_Object *eo,
																	void *ev_data __UNUSED__)
{
   ecdb_filelist_focus_handle(OUT, eo);
}

static void
_cb_filelist_mouse_down(void *data, Evas *e __UNUSED__, Evas_Object *eo,
																	void *ev_data __UNUSED__)
{
   Evas_Object *gui = data;
   evas_object_focus_set(eo, TRUE);

   edje_object_signal_emit(gui, "ecdb,filelist_overlay,deactivate", "ecdb");
   edje_object_part_text_set(gui, "filelist_overlay_text", NULL);

}

static void
_cb_filelist_key_down(void *data, Evas *e __UNUSED__,
            Evas_Object *eo __UNUSED__, void *ev_data)
{
   Evas_Event_Key_Down *ek = ev_data;
   Evas_Object *gui = data;

   if (!strcmp(ek->key, "Escape"))
   {
      edje_object_signal_emit(gui, "ecdb,filelist_overlay,deactivate", "ecdb");
      edje_object_part_text_set(gui, "filelist_overlay_text", NULL);
   }

   else if (!strcmp(ek->key, "Return"))
   {
      ecdb_handle_typebuf(gui);
   }

   else if (!strcmp(ek->key, "BackSpace"))
   {
      int len;
      const char *t;
      char *text;

      t = edje_object_part_text_get(gui, "filelist_overlay_text");
      text = (t) ? (strdup(t)) : NULL;

      if ((!text) || (!*text)) return;

      len = strlen(text);
      text = realloc(text, len);
      text[len - 1] = '\0';

      if (len == 1)
      {
         edje_object_signal_emit(gui, "ecdb,filelist_overlay,deactivate",
																							"ecdb");
      }
      edje_object_part_text_set(gui, "filelist_overlay_text",  text);
      FREE(text);
   }

   else 
   {
      int len;
      const char *t, *append;
      char *text;

      t = edje_object_part_text_get(gui, "filelist_overlay_text");

      text = (t) ? (strdup(t)) : NULL;

      if ((text) && (strlen(text) >= PATH_MAX))
         return;

      if ((ek->string) && (*ek->string) && (!ek->string[1]))
      {
         append = ek->string;
      }
      else if ((ek->keyname) && (*ek->keyname) && (!ek->key[1]))
      {
         append = ek->keyname;
      }
      else
      {
         if (text)
            free(text);
         return;
      }

      if (!text)
      {
         text = strdup(append);
      }
      else
      {
         len = strlen(text) + 2;
         text = realloc(text, len);
         ecore_strlcat(text, append, len);
      }
      
      /* Avoid sending the same signal twice... seems to skip the
       * transition in that case
       */
      if (strlen(text) == 1)
      {
         edje_object_signal_emit(gui, "ecdb,filelist_overlay,activate", "ecdb");
      }
      edje_object_part_text_set(gui, "filelist_overlay_text",  text);
      FREE(text);
   }
}

int
ecdb_create_main_gui(void)
{
   Evas_Object *gui;
   Evas_Coord mw, mh;
   const char *scale, *engine;
   float scalef;

   engine = getenv("ECDB_ENGINE");
   if (engine)
   {
      if (!strcmp(engine, "gl"))
         em->main_win_ee = ecore_evas_gl_x11_new(0, 0, 0, 0, 255, 255);
      else if (!strcmp(engine, "xr"))
         em->main_win_ee = ecore_evas_xrender_x11_new(0, 0, 0, 0, 255, 255);
      else if (!strcmp(engine, "x11"))
         em->main_win_ee = ecore_evas_software_x11_new(0, 0, 0, 0, 255, 255);
   }
   else
      em->main_win_ee = ecore_evas_software_x11_new(0, 0, 0, 0, 255, 255);

   if (!em->main_win_ee)
   {
      printf("Cannot create main window!\n");
      return FALSE;
   }

   ecore_evas_title_set(em->main_win_ee, "ECDB");
   ecore_evas_name_class_set(em->main_win_ee, "ECDB", "ECDB");
   ecore_evas_avoid_damage_set(em->main_win_ee, 1);
   ecore_x_dnd_aware_set(ecore_evas_software_x11_window_get(em->main_win_ee),
																				1);
   ecore_x_dnd_type_set(ecore_evas_software_x11_window_get(em->main_win_ee),
																				"*", 1);

   ecore_evas_callback_delete_request_set(em->main_win_ee, ecdb_shutdown);
   ecore_evas_callback_destroy_set(em->main_win_ee, ecdb_shutdown);

   ecore_evas_callback_mouse_in_set(em->main_win_ee, ecdb_cb_enter);
   ecore_evas_callback_mouse_out_set(em->main_win_ee, ecdb_cb_leave);
   ecore_evas_callback_resize_set(em->main_win_ee, ecdb_cb_resize);

   edje_frametime_set(1.0 / 60.0);
   ecore_evas_show(em->main_win_ee);

   /* Make this configurable at some point */
   ecdb_set_main_theme(NULL);
   scale = getenv("ECDB_SCALE");
   scalef = (scale) ? atof(scale) : 1.0;
   edje_scale_set(scalef);

   gui = edje_object_add(ecore_evas_get(em->main_win_ee));
   edje_object_file_set(gui, em->theme_path, "ecdb/window");
   edje_object_size_min_get(gui, &mw, &mh);
   if (mw <= 0) mw = 400;
   if (mh <= 0) mh = 300;
   mw = (int)((float)mw * scalef);
   mh = (int)((float)mh * scalef);
   evas_object_resize(gui, mw, mh);
   ecore_evas_resize(em->main_win_ee, mw, mh);
   ecore_evas_size_min_set(em->main_win_ee, mw, mh);
   evas_object_name_set(gui, "gui");
   evas_object_move(gui, 0, 0);
   evas_object_show(gui);

   ecdb_welcome_page_show();
   return TRUE;
}

/* This needs some work.  Where should user themes go? */
void
ecdb_set_main_theme(const char *theme_name)
{
   if (!theme_name)
   {
      snprintf(em->theme_path, PATH_MAX, "%s/%s", PACKAGE_DATA_DIR,
																		"themes/default.edj");
      evas_font_path_append(ecore_evas_get(em->main_win_ee),
																		PACKAGE_DATA_DIR"/font");
   }
}

void
ecdb_handle_typebuf(Evas_Object *gui)
{
   Ewl_Widget *fl;
   Ewl_Filelist_Filter *filter;
   const char *ext;

   if (!(ext = edje_object_part_text_get(gui, "filelist_overlay_text")))
   {
      return;
   }
   
   fl = ewl_widget_name_find("main_filelist");
   
   /* Check for a 'cd' first */
   if (!ecdb_match_keyword(ext, "cd", 2))
   {
      char *dir;

      /* What to do about multi-word directories? */
      dir = ecdb_strip_string(ext);

      if (dir)
      {
         if (dir[0] == '/')
         {
            if (ecore_file_exists(dir))
			   {
               ewl_filelist_directory_set(EWL_FILELIST(fl), dir);
			   }
         }
         else if (dir[0] == '~')
         {
            char path[PATH_MAX];

            snprintf(path, PATH_MAX, "%s%s", getenv("HOME"), &dir[1]);

            if (ecore_file_exists(path))
			   {
               ewl_filelist_directory_set(EWL_FILELIST(fl), path);
			   }
         }
         else
         {
            /* Try to concate it to our current directory */
            char path[PATH_MAX];

            snprintf(path, PATH_MAX, "%s/%s", ewl_filelist_directory_get
																		(EWL_FILELIST(fl)), dir);

            if (ecore_file_exists(path))
			   {
               ewl_filelist_directory_set(EWL_FILELIST(fl), path);
			   }
         }

         FREE(dir);
      }

      edje_object_signal_emit(gui, "ecdb,filelist_overlay,deactivate", "ecdb");
      edje_object_part_text_set(gui, "filelist_overlay_text", NULL);
      return;
   }
   else if (!ecdb_match_keyword(ext, "exit", 4))
   {
      /* This creates an ewl error... */
      ecore_event_add(ECORE_EVENT_SIGNAL_EXIT, NULL, NULL, NULL);
      return;
   }
   
   filter = ewl_filelist_filter_get(EWL_FILELIST(fl));
   if (!filter)
   {
      filter = calloc(sizeof(Ewl_Filelist_Filter), 1);
   }
   
   if (filter->extension)
   {
      if (strcmp(filter->extension, ext))
      {
         FREE(filter->extension);
      }
      else
      {
         return;
      }
   }
   
   filter->extension = strdup(ext);
   ewl_filelist_filter_set(EWL_FILELIST(fl), filter);
   ewl_filelist_refresh(EWL_FILELIST(fl));
   edje_object_signal_emit(gui, "ecdb,filelist_overlay,deactivate", "ecdb");
   edje_object_part_text_set(gui, "filelist_overlay_text", NULL);
}

static void
_button_cb_burn_data(void *data __UNUSED__, Evas_Object *o,
												const char *emission __UNUSED__,
                                    const char *source __UNUSED__)
{
   edje_object_signal_emit(o, "ecdb,welcome_page,hide", "ecdb");
   ecdb_burn_data_page_show();
   ecdb_filelist_show();
}

static void
_button_cb_erase(void *data __UNUSED__, Evas_Object *o,
												const char *emission __UNUSED__,
                                    const char *source __UNUSED__)
{
   edje_object_signal_emit(o, "ecdb,welcome_page,hide", "ecdb");
   ecdb_erase_page_show();
}

static void
_button_cb_burn_image(void *data __UNUSED__, Evas_Object *o,
												const char *emission __UNUSED__,
                                    const char *source __UNUSED__)
{
   edje_object_signal_emit(o, "ecdb,welcome_page,hide", "ecdb");
   ecdb_burn_image_page_show();
   ecdb_filelist_show();
}


static void
_button_cb_burn_audio(void *data __UNUSED__, Evas_Object *o,
												const char *emission __UNUSED__,
                                    const char *source)
{
   printf("Action: %s\n", source);
}

void
ecdb_welcome_page_show(void)
{
   Evas_Object *gui, *swallow;

   gui = evas_object_name_find(ecore_evas_get(em->main_win_ee), "gui");
   swallow = evas_object_name_find(ecore_evas_get(em->main_win_ee),
																					"welcome_page");

   if (!swallow)
   {

      Evas_Coord x, y, w, h;

      swallow = edje_object_add(ecore_evas_get(em->main_win_ee));
      evas_object_name_set(swallow, "welcome_page");
      edje_object_file_set(swallow, em->theme_path, "ecdb/welcome_page");
      edje_object_part_geometry_get(gui, "active_area", &x, &y, &w, &h);
      evas_object_move(swallow, x, y);
      evas_object_resize(swallow, w, h);
      edje_object_part_swallow(gui, "action_area", swallow);
      edje_object_signal_callback_add(swallow, "hide,finished", "welcome_page",
												ecdb_page_hide_finished, NULL);
      evas_object_show(swallow);

      edje_object_part_text_set(swallow, "ecdb/burn_data/text", "Burn Data CD");
      edje_object_signal_callback_add(swallow, "ecdb/burn_data,clicked", "*",
                                             _button_cb_burn_data, NULL);
      edje_object_part_text_set(swallow, "ecdb/burn_audio/text",
                                                            "Burn Audio CD");
      edje_object_signal_callback_add(swallow, "ecdb/burn_audio,clicked", "*",
                                             _button_cb_burn_audio, NULL);
      edje_object_part_text_set(swallow, "ecdb/burn_image/text", "Burn Image");
      edje_object_signal_callback_add(swallow, "ecdb/burn_image,clicked", "*",
                                             _button_cb_burn_image, NULL);
      edje_object_part_text_set(swallow, "ecdb/erase/text",
                                                "Erase Re-writable Disc");
      edje_object_signal_callback_add(swallow, "ecdb/erase,clicked", "*",
                                             _button_cb_erase, NULL);
   }

   else if (edje_object_part_swallow_get(gui, "action_area") != swallow)
   {
      /* All that is needed here? */
      edje_object_part_swallow(gui, "action_area", swallow);
      evas_object_show(swallow);
   }

   edje_object_signal_emit(gui, "ecdb,welcome_page,show", "ecdb");
   edje_object_signal_emit(swallow, "ecdb,welcome_page,show", "ecdb");
}

static void
ecdb_filelist_show(void)
{
   Evas_Object *gui, *swallow;

   gui = evas_object_name_find(ecore_evas_get(em->main_win_ee), "gui");
   swallow = evas_object_name_find(ecore_evas_get(em->main_win_ee),
																				"main_filelist");
   
   if (!swallow)
   {
      Evas_Coord x, y, w, h;
      Ewl_Widget *embed, *filelist;

      embed = ewl_embed_new();
      ewl_object_fill_policy_set(EWL_OBJECT(embed), EWL_FLAG_FILL_ALL);
      swallow = ewl_embed_canvas_set(EWL_EMBED(embed),
		 ecore_evas_get(em->main_win_ee),
            (void *)(long)ecore_evas_software_x11_window_get(em->main_win_ee));
      ewl_embed_focus_set(EWL_EMBED(embed), TRUE);
      ewl_widget_show(embed);

      filelist = ewl_filelist_new();
      ewl_widget_name_set(EWL_WIDGET(filelist), "main_filelist");
      ewl_container_child_append(EWL_CONTAINER(embed), filelist);
      ewl_filelist_directory_set(EWL_FILELIST(filelist), getenv("HOME"));
      ewl_widget_show(filelist);
   
      edje_object_part_geometry_get(gui, "filelist", &x, &y, &w, &h);
      evas_object_name_set(swallow, "main_filelist");
      evas_object_move(swallow, x, y);
      evas_object_resize(swallow, w, h);
      edje_object_part_swallow(gui, "filelist", swallow);
      evas_object_event_callback_add(swallow, EVAS_CALLBACK_MOUSE_DOWN,
															_cb_filelist_mouse_down, gui);
      evas_object_event_callback_add(swallow, EVAS_CALLBACK_KEY_DOWN,
															_cb_filelist_key_down, gui);
      evas_object_event_callback_add(swallow, EVAS_CALLBACK_MOUSE_IN,
                                             _cb_filelist_mouse_in, gui);
      evas_object_event_callback_add(swallow, EVAS_CALLBACK_MOUSE_OUT,
                                             _cb_filelist_mouse_out, gui);
      evas_object_show(swallow);
   }

   /* Assume here that it is the active projects job to hide itself and
    * call this show function
    */
   else if (edje_object_part_swallow_get(gui, "filelist") != swallow)
   {
      /* Is this all that is neccessary? */
      edje_object_part_swallow(gui, "filelist", swallow);
      evas_object_show(swallow);
   }

   edje_object_signal_emit(gui, "ecdb,filelist,visible", "ecdb");
   edje_object_signal_emit(swallow, "ecdb,filelist,visible", "ecdb");
}


/* This should be triggered by the theme, once it is done with its
 * nice fancy animations
 */
static void
ecdb_page_hide(const char *pname)
{
   Evas_Object *swallow, *gui;

   swallow = evas_object_name_find(ecore_evas_get(em->main_win_ee), pname);
   gui = evas_object_name_find(ecore_evas_get(em->main_win_ee), "gui");

   if (gui)
   {
      edje_object_part_unswallow(gui, swallow);
      evas_object_hide(swallow);
   }
}

/* Hurrah! Fancyness */
void
ecdb_page_hide_finished(void *data, Evas_Object *o, const char *emission,
                     const char *source)
{
   ecdb_page_hide(source);
}

void
ecdb_gui_controls_disable(const char **ids, int n)
{
   Evas_Object *obj;
   int i;

   for (i = 0; i < n; i++)
   {
      obj = evas_object_name_find(ecore_evas_get(em->main_win_ee), ids[i]);
      edje_object_signal_emit(obj, "ecdb,disable", "ecdb");
   }
}

void
ecdb_gui_controls_enable(const char **ids, int n)
{
   Evas_Object *obj;
   int i;

   for (i = 0; i < n; i++)
   {
      obj = evas_object_name_find(ecore_evas_get(em->main_win_ee), ids[i]);
      edje_object_signal_emit(obj, "ecdb,enable", "ecdb");
   }
}

void
ecdb_gui_combo_header_from_drive(Evas_Object *c, const char *name, void *data,
                                                   Evas_Object *obj, int sel)
{
   Evas_Object *ret;
   Ecdb_Drive_Info *drive;

   ret = ecdb_label_add(c, name);
   drive = eina_list_nth(em->drives, sel);
   ecdb_label_text_set(ret, drive->product);
   evas_object_show(ret);
}

void
ecdb_gui_combo_header_from_speeds(Evas_Object *c, const char *name, void *data,
                                                Evas_Object *obj, int sel)
{
   Evas_Object *ret;
   Ecdb_Drive_Info *drive;
   char buf[1024];

   drive = data;

   if (!drive)
      return;

   ret = ecdb_label_add(c, name);
   snprintf(buf, sizeof(buf), "%d", ecdb_speed_convert(drive, sel));
   ecdb_label_text_set(ret, buf);
   evas_object_show(ret);
}

